/*
   SSHSecure - a program to harden OpenSSH from defaults
   Copyright (C) 2020  Brent Saner

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

package sshkeys

import (
	"crypto/rand"
	"crypto/rsa"
)

func (k *SSHPrivKey) generateRsa() error {
	if k.BitSize == 0 {
		k.BitSize = defRSABitSize
	}
	if k.Key != nil || k.PublicKey.Key != nil {
		return nil // A no-op; key already exists.
	}
	if sk, err := rsa.GenerateKey(rand.Reader, int(k.BitSize)); err != nil {
		return err
	} else {
		k.Key = sk // See https://golang.org/pkg/crypto/rsa/#PrivateKey
		k.PublicKey.KeyType = KeyRsa
		k.PublicKey.Key = sk.PublicKey
	}
	return nil
}
